
/*
 * This file is part of IndustryStabilizer, which is a GameScript for OpenTTD
 *
 * IndustryStabilizer is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * IndustryStabilizer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with IndustryStabilizer; If not, see <http://www.gnu.org/licenses/> or
 * write to the Free Software Foundation, Inc., 51 Franklin Street, 
 * Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
 
  // Contributors:



/* Import SuperLib for GameScript */
import("util.superlib", "SuperLib", 29);
Result <- SuperLib.Result;
Log <- SuperLib.Log;
//Helper <- SuperLib.Helper;
ScoreList <- SuperLib.ScoreList;
Tile <- SuperLib.Tile;
Direction <- SuperLib.Direction;

class IndustryStabilizer extends GSController 
{
	constructor()
	{
	this.mapsize = GSMap.GetMapSize() / 262144 + 1;
	// Get executive rights immediately.
	//mode = GSExecMode();
	}
	mapsize = 1;
	on_load = false;
	go = true;
	refresh = true;
	first_manage = true;
	// base production value for all cargo.
	base_production = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];
	// the industries that can produce cargo. 64 32-bit masks of the cargo produced by industry.
	// note: only raw industries are ever spawned by this script.
	industries_producing = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];
	function Init();
	}
	

// for testing sendstats
//require("sendstats.nut");	
	
	function IndustryStabilizer::Save()
{
	Log.Info("Saving data", Log.LVL_INFO);
	local data = {sv_base = base_production,
	sv_produce = industries_producing
};
	return data;
}

function IndustryStabilizer::Load(version, tbl)
{
Log.Info("Loading data from savegame made with version " + version + " of the game script", Log.LVL_INFO);
foreach(key, val in tbl)
	{	
	if(key == "sv_base") 					base_production = val;	
	if(key == "sv_produce") 				industries_producing = val;
	//////////////////////////////////////////////////// SIMPLE VARS
	}
	///////////////////////////////////////////////// COMPLEX VARS
foreach(key, val in tbl)
	{	
	}
this.on_load = true;
}

function IndustryStabilizer::Second_Load(){

}

//////////////////////////////////// Program entry point
function IndustryStabilizer::Start()
{
	GSGame.Pause();
	this.Init();
	GSGame.Unpause();

while (go == true) 
	{
	this.Manage();
	}
	EndGame();
}
//////////////////////////////////// If you want to use IS as part of another script, 
// Call this function on game start, and 
// Connect the load and save functions, and
// Make sure to call my Manage() at least once per day.
function IndustryStabilizer::Start_Lib()
{
	GSGame.Pause();
	this.Init();
	GSGame.Unpause();
}
/////////////////////////////////// Main game loop
	function IndustryStabilizer::Manage()
	
{
	if(GSDate.GetCurrentDate() % GSController.GetSetting("rebuild_speed") == 0 && refresh)
	{
	local IndustryList = GSIndustryList();
	local current_production = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];
	foreach(Industry,_ in IndustryList)
		{
		for(local i = 0; i < 32; ++i)
			{
			current_production[i] += GSIndustry.GetLastMonthProduction(Industry, i);
			}
		}
	if(GSController.GetSetting("debug_level") > 0)
		{
		Log.Info("Total Map Production: ", Log.LVL_INFO); 
		for(local i = 0; i < 32; ++i)
			{
			if(current_production[i] > 0){
				GSLog.Info(GSCargo.GetCargoLabel(i)+": "+current_production[i]+ " | "+base_production[i]);
				}
			}
		}	
	//////////// production handlers	
	local OldConstr = GSGameSettings.GetValue("construction.raw_industry_construction");
	GSGameSettings.SetValue("construction.raw_industry_construction", 2);
	for(local i = 0; i < 32; ++i)
		{
			local delta = base_production[i] * GSController.GetSetting("bb_percentage") - current_production[i] * 100;
			if( delta > 0 && base_production[i] > 0)
			{
				// now we want to prospect some extra industry!
				// we want 1 industry per 1000 cargo units difference
				for(local j = 0; j <= delta/100000; ++j)
				{
				local IndustryType = SelectIndustryType(i);
				if(IndustryType != -1) GSIndustryType.ProspectIndustry(IndustryType);
				if(GSController.GetSetting("debug_level") > 0)
					{
					GSLog.Info("Building A "+GSIndustryType.GetName(IndustryType));
					}
				}
			}
		}
	GSGameSettings.SetValue("construction.raw_industry_construction", OldConstr);
	refresh = false;
	}
else{
	Sleep(1);
	}
if(GSDate.GetCurrentDate() % GSController.GetSetting("rebuild_speed") != 0)
	refresh = true;
}

function IndustryStabilizer::SelectIndustryType(cargotype){
// determine which types produce this cargo
local types = [];
for(local i = 0; i < industries_producing.len(); ++i)
	{
	if(industries_producing[i] & (1 << cargotype))
	types.append(i);
	}	
	if(types == []) return -1;
return types[GSBase.RandRange(types.len())];
}

////////////////////////////// Initialization function.
function IndustryStabilizer::Init(){
local Cargolist;
local IndustryTypeList = GSIndustryTypeList();
foreach(IType,_ in IndustryTypeList)
	{
			GSLog.Info("Industry:" + IType);
	if (GSIndustryType.IsRawIndustry(IType))
		{
		Cargolist = GSIndustryType.GetProducedCargo(IType);
		foreach(cargo,_ in Cargolist)
			{
			GSLog.Info("Cargo" + cargo + ", Industry:" + IType);
			industries_producing[IType] += 1<<cargo;
			}
		}
	}
if(GSController.GetSetting("debug_level") > 0)
	{
	GSLog.Info("IndustryProdList: "); 
	foreach(ind in industries_producing)
		{
		GSLog.Info("P" + ind);
		}
	}
	
// first get the current industry list
local IndustryList = GSIndustryList();
// 
foreach(Industry,_ in IndustryList)
	{
	for(local i = 0; i < 32; ++i)
		{
	 	base_production[i] += GSIndustry.GetLastMonthProduction(Industry, i);
		}
	}
if(GSController.GetSetting("debug_level") > 0)
	{
	GSLog.Info("Total Map Production: "); 
	for(local i = 0; i < 32; ++i)
		{
			if(base_production[i] > 0){
			GSLog.Info(GSCargo.GetCargoLabel(i)+": "+base_production[i]);}
		}
	}


}
// NOTE: Has to be called from the HandleEvents of main.
function IndustryStabilizer::HandleEvents()
{
	while(GSEventController.IsEventWaiting())
	{
		local ev = GSEventController.GetNextEvent();
		
		switch(ev)
		{
		}

		if(ev == null)
			return;
	}
}
